import os
from ast import literal_eval
import pdb
import json
import requests
from tqdm import tqdm


class SearchAPI():
    def __init__(self):
        # invariant variables
        self.serper_key = os.getenv("SERPER_KEY_PRIVATE")
        self.url = "https://google.serper.dev/search"
        self.headers = {'X-API-KEY': self.serper_key,
                        'Content-Type': 'application/json'}
        # cache related
        self.cache_file = "data/cache/search_cache.json"
        self.cache_dict = self.load_cache()
        self.add_n = 0
        self.save_interval = 10

    def get_snippets(self, claim_lst):
        text_claim_snippets_dict = {}
        for query in claim_lst:
            search_result = self.get_search_res(query)
            if "statusCode" in search_result:  # and search_result['statusCode'] == 403:
                print(search_result['message'])
                exit()
            if "organic" in search_result:
                organic_res = search_result["organic"]
            else:
                organic_res = []

            search_res_lst = []
            for item in organic_res:
                title = item["title"] if "title" in item else ""
                snippet = item["snippet"] if "snippet" in item else ""
                link = item["link"] if "link" in item else ""

                search_res_lst.append({"title": title,
                                       "snippet": snippet,
                                       "link": link})
            text_claim_snippets_dict[query] = search_res_lst
        return text_claim_snippets_dict

    def get_search_res(self, query):
        # check if prompt is in cache; if so, return from cache
        cache_key = query.strip()
        if cache_key in self.cache_dict:
            # print("Getting search results from cache ...")
            return self.cache_dict[cache_key]

        payload = json.dumps({"q": query})
        response = requests.request("POST",
                                    self.url,
                                    headers=self.headers,
                                    data=payload)
        response_json = literal_eval(response.text)

        # update cache
        self.cache_dict[query.strip()] = response_json
        self.add_n += 1

        # save cache every save_interval times
        if self.add_n % self.save_interval == 0:
            self.save_cache()

        return response_json

    def save_cache(self):
        # load the latest cache first, since if there were other processes running in parallel, cache might have been updated
        cache = self.load_cache().items()
        for k, v in cache:
            self.cache_dict[k] = v
        print(f"Saving search cache ...")
        with open(self.cache_file, "w") as f:
            json.dump(self.cache_dict, f, indent=4)

    def load_cache(self):
        if os.path.exists(self.cache_file):
            with open(self.cache_file, "r") as f:
                # load a json file
                cache = json.load(f)
                print(f"Loading cache ...")
        else:
            cache = {}
        return cache



import os
import pdb
import json
import random
import argparse
from tqdm import tqdm
from .search_API import SearchAPI

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument("--data_dir", type=str, default='./data')
    parser.add_argument("--input_file", type=str, required=True)
    parser.add_argument("--output_dir", type=str, default='./data')
    args = parser.parse_args()

    """
    Get search results for each claim
    Store as a dictionary {claim: {"search_results": [search_results]}}
    """
    # initialize search api
    fetch_search = SearchAPI()

    input_file_name = "".join(args.input_file.split('.')[:-1])
    input_path = os.path.join(args.data_dir, args.input_file)

    # read in extracted claim data
    with open(input_path, "r") as f:
        data = [json.loads(x) for x in f.readlines()]

    output_dir = args.output_dir
    output_file = f"evidence_{input_file_name}.jsonl"
    output_path = os.path.join(output_dir, output_file)
    os.makedirs(os.path.dirname(output_dir), exist_ok=True)

    with open(output_path, "w") as f:
        for dict_item in tqdm(data):
            if dict_item['abstained']:
                f.write(json.dumps(dict_item) + "\n")
                continue

            claim_lst = dict_item["all_claims"]
            if claim_lst == ["No verifiable claim."]:
                dict_item["claim_search_results"] = []
                f.write(json.dumps(dict_item) + "\n")
                continue
            claim_snippets = fetch_search.get_snippets(claim_lst)
            dict_item["claim_search_results"] = claim_snippets

            f.write(json.dumps(dict_item) + "\n")
            f.flush()



import os
import pdb
import json
import tiktoken
from openai import OpenAI
from anthropic import Anthropic


class GetResponse():
    # OpenAI model list: https://platform.openai.com/docs/models/gpt-4-and-gpt-4-turbo
    # Claude3 model list: https://www.anthropic.com/claude
    # "claude-3-opus-20240229", gpt-4-0125-preview
    def __init__(self, cache_file, model_name="gpt-4-0125-preview", max_tokens=1000, temperature=0):
        self.model_name = model_name
        self.max_tokens = max_tokens
        self.temperature = temperature
        self.cache_file = cache_file

        # invariant variables
        self.tokenizer = tiktoken.encoding_for_model("gpt-4")
        # model keys
        if "gpt" in model_name:
            self.key = os.getenv("OPENAI_API_KEY_PERSONAL")
            self.client = OpenAI(api_key=self.key)
            self.seed = 1130
        elif "claude" in model_name:
            self.key = os.getenv("CLAUDE_API_KEY")
            self.client = Anthropic(api_key=self.key)
        # cache related
        self.cache_dict = self.load_cache()
        self.add_n = 0
        self.save_interval = 20

    # Returns the response from the model given a system message and a prompt text.
    def get_response(self, system_message, prompt_text, cost_estimate_only=False):
        prompt_tokens = len(self.tokenizer.encode(prompt_text))
        if cost_estimate_only:
            # count tokens in prompt and response
            response_tokens = 0
            return None, prompt_tokens, response_tokens

        # check if prompt is in cache; if so, return from cache
        cache_key = prompt_text.strip()
        if cache_key in self.cache_dict:
            return self.cache_dict[cache_key], 0, 0

        # Example message: "You are a helpful assistant who can extract verifiable atomic claims from a piece of text."
        if "gpt" in self.model_name:
            message = [{"role": "system", "content": system_message},
                       {"role": "user", "content": prompt_text}]
            response = self.client.chat.completions.create(
                model=self.model_name,
                messages=message,
                max_tokens=self.max_tokens,
                temperature=self.temperature,
                seed=self.seed,
            )
            response_content = response.choices[0].message.content.strip()
        elif "claude" in self.model_name:
            response = self.client.messages.create(
                model=self.model_name,
                messages=[{"role": "user", "content": prompt_text}],
                temperature=self.temperature,
                max_tokens=self.max_tokens
            )
            response_content = response.content[0].text.strip()

        # update cache
        self.cache_dict[cache_key] = response_content.strip()
        self.add_n += 1

        # save cache every save_interval times
        if self.add_n % self.save_interval == 0:
            self.save_cache()

        response_tokens = len(self.tokenizer.encode(response_content))
        return response_content, prompt_tokens, response_tokens

    # Returns the number of tokens in a text string.
    def tok_count(self, text: str) -> int:
        num_tokens = len(self.tokenizer.encode(text))
        return num_tokens

    def save_cache(self):
        # load the latest cache first, since if there were other processes running in parallel, cache might have been updated
        for k, v in self.load_cache().items():
            self.cache_dict[k] = v
        print(f"Saving cache to {self.cache_file}...")
        with open(self.cache_file, "w") as f:
            json.dump(self.cache_dict, f, indent=4)

    def load_cache(self):
        if os.path.exists(self.cache_file):
            with open(self.cache_file, "r") as f:
                # load a json file
                cache = json.load(f)
                print(f"Loading cache from {self.cache_file}...")
        else:
            cache = {}
        return cache



import os
import json
import argparse
from tqdm import tqdm
from .claim_verifier import ClaimVerifier

if __name__ == '__main__':
    """
    add argparse for label_n and model_name
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("--data_dir", type=str, default='./data')
    parser.add_argument("--input_file", type=str, required=True)
    parser.add_argument("--output_dir", type=str, default='./data')
    parser.add_argument("--cache_dir", type=str, default='./data/cache')
    parser.add_argument("--model_name", type=str, default="gpt-4o")
    parser.add_argument("--label_n", type=int, default=3, choices=[2, 3])
    parser.add_argument("--search_res_num", type=int, default=5)
    args = parser.parse_args()

    model_name = args.model_name
    label_n = args.label_n

    input_file_name = "".join(args.input_file.split('.')[:-1])
    input_path = os.path.join(args.data_dir, args.input_file)
    with open(input_path, "r") as f:
        input_data = [json.loads(line) for line in f if line.strip()]
    demon_dir = os.path.join(args.data_dir, 'demos')
    claim_verifier = ClaimVerifier(model_name=model_name, label_n=label_n,
                                   cache_dir=args.cache_dir, demon_dir=demon_dir)

    output_dir = os.path.join(args.output_dir, 'model_output')
    os.makedirs(output_dir, exist_ok=True)
    output_file = f'verification_{input_file_name}_{label_n}.jsonl'
    output_path = os.path.join(output_dir, output_file)

    scores = []
    total_prompt_tok_cnt = 0
    total_resp_tok_cnt = 0

    with open(output_path, "w") as f:
        for dict_item in tqdm(input_data):
            claim_search_results = dict_item["claim_search_results"]

            if dict_item['abstained']:
                f.write(json.dumps(dict_item) + "\n")
                continue

            if not claim_search_results:
                scores.append(0)

            claim_verify_res_dict, prompt_tok_cnt, response_tok_cnt = claim_verifier.verifying_claim(
                claim_search_results, search_res_num=args.search_res_num)

            # get the supported claims
            supported_claims = []
            for claim, res in claim_verify_res_dict.items():
                model_decision = res['verification_result']
                if model_decision == "supported":
                    supported_claims.append(claim)

            # get sentence ave
            sent_score_lst = []
            for claim_lst in dict_item['claim_list']:
                sent_numerator = 0
                sent_denominator = len(claim_lst)
                for claim in claim_lst:
                    if claim in supported_claims:
                        sent_numerator += 1
                sent_score = sent_numerator / sent_denominator
                sent_score_lst.append(sent_score)

            resp_score = sum(sent_score_lst) / len(sent_score_lst)
            scores.append(resp_score)

            f.write(json.dumps(claim_verify_res_dict) + "\n")
            total_prompt_tok_cnt += prompt_tok_cnt
            total_resp_tok_cnt += response_tok_cnt
    print(
        f"\tScore: {sum(scores) / len(scores):.2f}")
    print(f"Total cost: {total_prompt_tok_cnt * 10 / 1e6 + total_resp_tok_cnt * 30 / 1e6}")



from setuptools import setup, find_packages

setup(
   name="VeriScore",
   version="1.0.4",
   packages=find_packages(),
   install_requires=[
       #'requests', 'numpy'
      'spacy',
      'openai',
      'anthropic',
      'tiktoken',
      'tqdm',
   ],
   description="Pip package for Verifact",
   long_description=open('README.md').read(),
   long_description_content_type="text/markdown",
   classifiers=[
       "Programming Language :: Python :: 3",
       "License :: OSI Approved :: MIT License",
       "Operating System :: OS Independent",
   ],
   python_requires='>=3.9',
)



"""
This script is written to extract claims from the model responses.
model generations: /data/yixiao/atomic_claims/data/model_generation_decomposition/model_generations
"""

import os
import json
import argparse
import spacy
from tqdm import tqdm
from .claim_extractor import ClaimExtractor
from .search_API import SearchAPI
from .claim_verifier import ClaimVerifier

abstain_responses = ["I'm sorry, I cannot fulfill that request.",
                     "I'm sorry, I can't fulfill that request.",
                     "I'm sorry, but I cannot fulfill that request.",
                     "I'm sorry, but I can't fulfill that request.",
                     "Sorry, but I can't fulfill that request.",
                     "Sorry, I can't do that."]

class VeriScorer(object):
    def __init__(self,
                 model_name_extraction='gpt-4-0125-preview',
                 model_name_verification='gpt-4o',
                 data_dir='./data',
                 cache_dir='./data/cache',
                 output_dir='./data_cache',
                 label_n=3,
                 search_res_num=5):
        self.data_dir = data_dir
        self.output_dir = output_dir
        self.cache_dir = cache_dir

        os.makedirs(output_dir, exist_ok=True)
        os.makedirs(self.cache_dir, exist_ok=True)

        self.spacy_nlp = spacy.load('en_core_web_sm')

        self.system_message_extraction = "You are a helpful assistant who can extract verifiable atomic claims from a piece of text. Each atomic fact should be verifiable against reliable external world knowledge (e.g., via Wikipedia)"

        self.claim_extractor = ClaimExtractor(model_name_extraction, cache_dir=self.cache_dir)

        self.fetch_search = SearchAPI()

        demon_dir = os.path.join(self.data_dir, 'demos')
        self.model_name_verification = model_name_verification
        self.claim_verifier = ClaimVerifier(model_name=model_name_verification, label_n=label_n,
                                         cache_dir=self.cache_dir, demon_dir=demon_dir)
        self.label_n = label_n
        self.search_res_num = search_res_num

    def get_veriscore(self, data, input_file_name):

        ### extract claims ###
        output_file = f"claims_{input_file_name}.jsonl"
        output_path = os.path.join(self.output_dir, output_file)

        extracted_claims = []
        with open(output_path, "w") as f:
            for dict_item in tqdm(data):
                response = dict_item["response"]
                prompt_source = dict_item["prompt_source"]
                model = dict_item["model"]

                # skip abstained responses
                if response.strip() in abstain_responses:
                    output_dict = {"question": question.strip(),
                                   "response": response.strip(),
                                   "abstained": True,
                                   "prompt_source": prompt_source,
                                   "model": model, }
                    f.write(json.dumps(output_dict) + "\n")
                    continue

                if "question" in dict_item:
                    question = dict_item["question"]
                    snippet_lst, claim_list, all_claims, prompt_tok_cnt, response_tok_cnt = self.claim_extractor.qa_scanner_extractor(
                        question, response)
                else:
                    question = ''
                    snippet_lst, claim_list, all_claims, prompt_tok_cnt, response_tok_cnt = self.claim_extractor.non_qa_scanner_extractor(
                        response)

                # write output
                output_dict = {"question": question.strip(),
                               "prompt_source": prompt_source,
                               "response": response.strip(),
                               "prompt_tok_cnt": prompt_tok_cnt,
                               "response_tok_cnt": response_tok_cnt,
                               "model": model,
                               "abstained": False,  # "abstained": False, "abstained": True
                               "claim_list": claim_list,
                               "all_claims": all_claims
                               }
                f.write(json.dumps(output_dict) + "\n")
                extracted_claims.append(output_dict)

        print(f"claim extraction is done! saved to {output_path}")

        output_file = f"evidence_{input_file_name}.jsonl"
        output_path = os.path.join(self.output_dir, output_file)
        searched_evidence_dict = []
        with open(output_path, "w") as f:
            for dict_item in tqdm(extracted_claims):
                if dict_item['abstained']:
                    f.write(json.dumps(dict_item) + "\n")
                    searched_evidence_dict.append(dict_item)
                    continue

                claim_lst = dict_item["all_claims"]
                if claim_lst == ["No verifiable claim."]:
                    dict_item["claim_search_results"] = []
                    f.write(json.dumps(dict_item) + "\n")
                    searched_evidence_dict.append(dict_item)
                    continue
                claim_snippets = self.fetch_search.get_snippets(claim_lst)
                dict_item["claim_search_results"] = claim_snippets
                searched_evidence_dict.append(dict_item)
                f.write(json.dumps(dict_item) + "\n")
                f.flush()
        print(f"evidence searching is done! saved to {output_path}")

        output_dir = os.path.join(args.output_dir, 'model_output')
        os.makedirs(output_dir, exist_ok=True)
        output_file = f'verification_{input_file_name}_{self.label_n}.jsonl'
        output_path = os.path.join(output_dir, output_file)

        scores = []
        total_prompt_tok_cnt = 0
        total_resp_tok_cnt = 0

        with open(output_path, "w") as f:
            for dict_item in tqdm(searched_evidence_dict):
                claim_search_results = dict_item["claim_search_results"]
                # no verifiable claim for the whole model response
                if not claim_search_results:
                    scores.append(0)
                claim_verify_res_dict, prompt_tok_cnt, response_tok_cnt = self.claim_verifier.verifying_claim(
                    claim_search_results, search_res_num=args.search_res_num)

                # get the supported claims
                supported_claims = []
                for claim, res in claim_verify_res_dict.items():
                    model_decision = res['verification_result']
                    if model_decision == "supported":
                        supported_claims.append(claim)

                # get sentence ave
                sent_score_lst = []
                for claim_lst in dict_item['claim_list']:
                    sent_numerator = 0
                    sent_denominator = len(claim_lst)
                    for claim in claim_lst:
                        if claim in supported_claims:
                            sent_numerator += 1
                    sent_score = sent_numerator / sent_denominator
                    sent_score_lst.append(sent_score)

                resp_score = sum(sent_score_lst) / len(sent_score_lst)
                scores.append(resp_score)

                json.dump(claim_verify_res_dict, f)
                f.write("\n")
                total_prompt_tok_cnt += prompt_tok_cnt
                total_resp_tok_cnt += response_tok_cnt

        print(f"claim verification is done! saved to {output_path}")
        print(f"\tScore: {sum(scores) / len(scores):.2f}")
        print(f"Total cost: {total_prompt_tok_cnt * 10 / 1e6 + total_resp_tok_cnt * 30 / 1e6}")

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("--data_dir", type=str, default='./data')
    parser.add_argument("--input_file", type=str, required=True)
    parser.add_argument("--output_dir", type=str, default='./data')
    parser.add_argument("--cache_dir", type=str, default='./data/cache')
    parser.add_argument("--model_name_extraction", type=str, default="gpt-4-0125-preview")
    parser.add_argument("--model_name_verification", type=str, default="gpt-4o")
    parser.add_argument("--label_n", type=int, default=3, choices=[2, 3])
    parser.add_argument("--search_res_num", type=int, default=5)
    args = parser.parse_args()


    vs = VeriScorer(model_name_extraction=args.model_name_extraction,
                    model_name_verification=args.model_name_verification,
                    data_dir=args.data_dir,
                    output_dir=args.output_dir,
                    cache_dir=args.cache_dir,
                    label_n=args.label_n,
                    search_res_num=args.search_res_num)

    input_file_name = "".join(args.input_file.split('.')[:-1])
    input_path = os.path.join(args.data_dir, args.input_file)
    with open(input_path, "r") as f:
        data = [json.loads(x) for x in f.readlines() if x.strip()]

    vs.get_veriscore(data, input_file_name)



import os
import regex
import pdb
import json
import spacy
from tqdm import tqdm
from .get_response import GetResponse


class ClaimExtractor():
    def __init__(self, model_name, cache_dir="./data/cache/"):
        self.model = None
        if os.path.isdir(model_name):
            from unsloth import FastLanguageModel
            self.model, self.tokenizer = FastLanguageModel.from_pretrained(
                model_name=model_name,
                max_seq_length=1024,
                dtype=None,
                load_in_4bit=True,
            )
            FastLanguageModel.for_inference(self.model)
            self.EOS_TOKEN = self.tokenizer.eos_token
            self.tokenizer.pad_token = "[PAD]"
            self.tokenizer.padding_side = "right"
            self.alpaca_prompt = open("./prompt/extraction_alpaca_template.txt", "r").read()
        else:
            cache_dir = os.path.join(cache_dir, model_name)
            os.makedirs(cache_dir, exist_ok=True)
            self.cache_file = os.path.join(cache_dir, f"claim_extraction_cache.json")
            self.get_model_response = GetResponse(cache_file=self.cache_file,
                                                  model_name=model_name,
                                                  max_tokens=1000,
                                                  temperature=0)
            self.system_message = "You are a helpful assistant who can extract verifiable atomic claims from a piece of text. Each atomic fact should be verifiable against reliable external world knowledge (e.g., via Wikipedia)"

        self.spacy_nlp = spacy.load('en_core_web_sm')

    def non_qa_scanner_extractor(self, response, cost_estimate_only=False):
        """
        Given a model output
        - split by \n into paragraphs
        - split the paragraphs into sentences using spaCy
        - go para by para, always add the first sent of the para into context1
        - snippet = (context1 = 0-3 sentence) <SOS>Sent<EOS> (context2 = 0-1 sentence)
        - call fact_extractor on each snippet
        """
        # split response into paras & clean out empty strings
        paragraph_lst = [x.strip() for x in response.split("\n") if x.strip() != ""]

        all_facts_lst = []
        # keep track of token counts
        prompt_tok_cnt, response_tok_cnt = 0, 0
        for para in paragraph_lst:
            # split the text into sentences using spaCy
            sentences = self.get_sentence(para)
            for i in range(len(sentences)):
                lead_sent = sentences[0]  # 1st sentence of the para
                context1 = " ".join(sentences[max(0, i - 3):i])
                sentence = f"<SOS>{sentences[i].strip()}<EOS>"
                context2 = " ".join(sentences[i + 1:i + 2])

                # if the para is not long
                if len(sentences) <= 5:
                    snippet = f"{context1.strip()} {sentence.strip()} {context2.strip()}".strip()
                # if the para is long, add lead sentence to context1
                else:
                    snippet = f"{lead_sent.strip()} {context1.strip()} {sentence.strip()} {context2.strip()}".strip()

                # call fact_extractor on each snippet
                facts, prompt_tok_num, response_tok_num = self.fact_extractor(snippet, sentences[i].strip(),
                                                                              qa_input=False,
                                                                              cost_estimate_only=cost_estimate_only)

                # update token counts
                prompt_tok_cnt += prompt_tok_num
                response_tok_cnt += response_tok_num

                if facts == None:
                    continue

                # deduplication
                for fact in facts:
                    if fact.strip() == "":
                        continue
                    # cases where GPT returns its justification
                    elif fact.startswith("Note:"):
                        continue
                    elif fact not in all_facts_lst:
                        all_facts_lst.append(fact)

        print(f"Returning facts and token counts for the whole response ...")
        if all_facts_lst == None:
            return None, prompt_tok_cnt, response_tok_cnt
        else:
            return all_facts_lst, prompt_tok_cnt, response_tok_cnt

    def qa_scanner_extractor(self, question, response, cost_estimate_only=False):
        """
        Given a model output to a question
        - split the response into sentences using spaCy
        - snippet = question (context1 = 0-3 sentence) <SOS>Sent<EOS> (context2 = 0-1 sentence)
        - call fact_extractor on each snippet
        """
        all_facts_lst = []
        # keep track of token counts
        prompt_tok_cnt, response_tok_cnt = 0, 0
        sentences = self.get_sentence(response)

        # new return values
        snippet_lst = []
        fact_lst_lst = []
        for i in range(len(sentences)):
            context1 = " ".join(sentences[max(0, i - 3):i])
            sentence = f"<SOS>{sentences[i].strip()}<EOS>"
            context2 = " ".join(sentences[i + 1:i + 2])

            snippet = f"Question: {question.strip()}\nResponse: {context1.strip()} {sentence.strip()} {context2.strip()}".strip()
            # new return value
            snippet_lst.append(snippet)

            # call fact_extractor on each tesnippetxt
            facts, prompt_tok_num, response_tok_num = self.fact_extractor(snippet, sentences[i].strip(), qa_input=True,
                                                                          cost_estimate_only=cost_estimate_only)

            # update token counts
            prompt_tok_cnt += prompt_tok_num
            response_tok_cnt += response_tok_num

            if facts == None:
                fact_lst_lst.append([None])
                continue

            # deduplication
            fact_lst = []
            for fact in facts:
                if fact.strip() == "":
                    continue
                # cases where GPT returns its justification
                elif fact.startswith("Note:"):
                    continue
                elif fact not in all_facts_lst:
                    all_facts_lst.append(fact.strip())
                fact_lst.append(fact.strip())
            fact_lst_lst.append(fact_lst)
        print(f"Returning facts and token counts for the whole response ...")

        return snippet_lst, fact_lst_lst, all_facts_lst, prompt_tok_cnt, response_tok_cnt

    def get_sentence(self, text):
        # use spaCy to split the text into sentences
        return [x.text.strip() for x in self.spacy_nlp(text).sents]

    def get_prompt_template(self, qa_input):
        if qa_input:
            prompt_template = open("./prompt/extraction_qa_template.txt", "r").read()
        else:
            prompt_template = open("./prompt/extraction_non_qa_template.txt", "r").read()
        return prompt_template

    def fact_extractor(self, snippet, sentence, qa_input=False, cost_estimate_only=False):
        """
        snippet = (context1) <SOS>sentence<EOS> (context2)
        sentence = the sentence to be focused on
        """

        if self.model:
            formatted_input = self.alpaca_prompt.format(input, "")
            inputs = self.tokenizer(formatted_input, return_tensors="pt").to("cuda")

            outputs = self.model.generate(**inputs, max_new_tokens=1000, use_cache=True)
            output_str = ' '.join(self.tokenizer.batch_decode(outputs))
            clean_output = output_str.split("### Response:")[-1].strip().replace("</s>", "")

            claims = [x.strip() for x in clean_output.split("\n")]
            return claims, 0, 0
        else:
            ### prompting base approach via API call
            prompt_template = self.get_prompt_template(qa_input)  # qa_prompt_temp if qa_input else non_qa_prompt_temp
            prompt_text = prompt_template.format(snippet=snippet, sentence=sentence)
            response, prompt_tok_cnt, response_tok_cnt = self.get_model_response.get_response(self.system_message,
                                                                                              prompt_text,
                                                                                              cost_estimate_only)

            if not response or "No verifiable claim." in response:
                return None, prompt_tok_cnt, response_tok_cnt
            else:
                # remove itemized list
                claims = [x.strip().replace("- ", "") for x in response.split("\n")]
                # remove numbers in the beginning
                claims = [regex.sub(r"^\d+\.?\s", "", x) for x in claims]
                return claims, prompt_tok_cnt, response_tok_cnt
